/**
 * Fichier d'entrée principal optimisé pour minimiser le JavaScript initial
 * Stratégie:
 * 1. N'importer que le strict minimum pour le rendu initial
 * 2. Charger de manière différée tout ce qui est non-critique
 * 3. Optimiser les imports React pour réduire le bundle vendor-react-core
 */

// Imports minimalistes pour le premier rendu
import { createRoot } from 'react-dom/client';
import App from './App.tsx';
// Styles critiques uniquement
import './critical.css';

// Chargement différé des polices pour réduire le CLS (Cumulative Layout Shift)
const checkFontsLoaded = () => {
  if ('fonts' in document) {
    document.fonts.ready.then(() => {
      document.documentElement.classList.add('fonts-loaded');
    });
  } else {
    setTimeout(() => {
      document.documentElement.classList.add('fonts-loaded');
    }, 300);
  }
};

// Chargement différé des styles non-critiques
const loadNonCriticalResources = () => {
  // Importer CSS non-critique via dynamic import
  import('./index.css').catch(err => 
    console.warn('Erreur de chargement des styles:', err)
  );
  import('./font.css').catch(err => 
    console.warn('Erreur de chargement des styles de police:', err)
  );
};

// Signal que le module PWA sera chargé dynamiquement
const registerServiceWorker = () => {
  import('virtual:pwa-register').then(({ registerSW }) => {
    const updateSW = registerSW({
      onNeedRefresh() {
        if (confirm('Une nouvelle version est disponible. Recharger pour mettre à jour?')) {
          updateSW(true);
        }
      },
      onOfflineReady() {
        if (process.env.NODE_ENV !== 'production') {
          console.log('L\'application est prête à fonctionner hors ligne');
        }
      },
    });
  });
};

// Appeler immédiatement la détection des polices pour le premier rendu
checkFontsLoaded();

// Création de la racine React avec rendering optimisé
// En supprimant StrictMode en production pour économiser du code
const rootElement = document.getElementById('root');
if (rootElement) {
  const root = createRoot(rootElement);
  
  if (process.env.NODE_ENV === 'production') {
    // En production, pas de StrictMode pour réduire la taille du bundle
    root.render(<App />);
  } else {
    // En développement uniquement, importer StrictMode à la demande
    import('react').then(({ StrictMode }) => {
      root.render(
        <StrictMode>
          <App />
        </StrictMode>
      );
    });
  }
  
  // Charger les styles non-critiques après le premier rendu
  loadNonCriticalResources();
}

// Chargement différé des services non critiques
const deferredInit = () => {
  // Import dynamique de Firebase
  import('./lib/firebase').then(({ initFirebase }) => {
    initFirebase().catch(error => {
      if (process.env.NODE_ENV !== 'production') {
        console.error('Erreur lors de l\'initialisation de Firebase:', error);
      }
    });
  });

  // Enregistrement du service worker
  registerServiceWorker();

  // Chargement d'autres scripts non critiques
  if (process.env.NODE_ENV === 'production') {
    // Analytiques et scripts tiers uniquement en production
    if ('requestIdleCallback' in window) {
      // Attendre que le navigateur soit inactif pour charger les scripts non critiques
      window.requestIdleCallback(() => {
        loadAnalytics();
      }, { timeout: 5000 });
    } else {
      // Fallback pour les navigateurs qui ne supportent pas requestIdleCallback
      setTimeout(() => {
        loadAnalytics();
      }, 5000);
    }
  }
};

// Fonction pour charger les scripts d'analytiques
const loadAnalytics = () => {
  // Exemples de scripts à charger de manière différée
  const scripts: string[] = [
    // Ajoutez ici d'autres scripts tiers si nécessaire
  ];

  scripts.forEach(src => {
    const script = document.createElement('script');
    script.src = src;
    script.async = true;
    script.defer = true;
    document.body.appendChild(script);
  });
};

// Initialisation différée après le premier rendu
if (document.readyState === 'complete') {
  deferredInit();
} else {
  window.addEventListener('load', deferredInit);
}
